# Generated by roxygen2: do not edit by hand

S3method(getClassProb,PREM)
S3method(getCoef,BEND)
S3method(getFitted,BEND)
S3method(getFixEf,BPREM)
S3method(getFixEf,CREM)
S3method(getFixEf,PREM)
S3method(getKProb,PREM)
S3method(getModelFit,BEND)
S3method(getRanEf,BPREM)
S3method(getRanEf,CREM)
S3method(getVarCov,BPREM)
S3method(getVarCov,CREM)
S3method(getVarCov,PREM)
S3method(plot,BPREM)
S3method(plot,CREM)
S3method(plot,PREM)
S3method(print,BPREM)
S3method(print,CREM)
S3method(print,PREM)
S3method(print,getClassProb.PREM)
S3method(print,getCoef)
S3method(print,getFitted)
S3method(print,getFixEf.BEND)
S3method(print,getKProb.PREM)
S3method(print,getModelFit.BEND)
S3method(print,getRanEf.BPREM)
S3method(print,getRanEf.CREM)
S3method(print,getVarCov.BPREM)
S3method(print,getVarCov.CREM)
S3method(print,getVarCov.PREM)
S3method(print,summary.BPREM)
S3method(print,summary.CREM)
S3method(print,summary.PREM)
S3method(summary,BPREM)
S3method(summary,CREM)
S3method(summary,PREM)
export(Bayes_BPREM)
export(Bayes_CREM)
export(Bayes_PREM)
export(getClassProb)
export(getCoef)
export(getFitted)
export(getFixEf)
export(getKProb)
export(getModelFit)
export(getRanEf)
export(getVarCov)
export(plot_BEND)
import(graphics)
import(stats)
