% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getModelFit.R
\name{getModelFit}
\alias{getModelFit}
\alias{getModelFit.BEND}
\alias{print.getModelFit.BEND}
\title{Extract model fit}
\usage{
getModelFit(x, ...)

\method{getModelFit}{BEND}(x, ...)

\method{print}{getModelFit.BEND}(x, ...)
}
\arguments{
\item{x}{An object of class "BPREM", "CREM", or "PREM".}

\item{...}{Additional arguments.}
}
\value{
Returns a vector of the model fit information (deviance, pD, DIC).
}
\description{
Extracts the model fit information from a fitted model of class "BPREM", "CREM", or "PREM".
}
\examples{
# load fitted model results
data(results_prem)
# get model fit
getModelFit(results_prem)

}
\author{
Corissa T. Rohloff
}
