% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getRanEf.R
\name{getRanEf}
\alias{getRanEf}
\alias{getRanEf.BPREM}
\alias{getRanEf.CREM}
\alias{print.getRanEf.BPREM}
\alias{print.getRanEf.CREM}
\title{Extract random effects}
\usage{
getRanEf(x, ...)

\method{getRanEf}{BPREM}(x, ...)

\method{getRanEf}{CREM}(x, ...)

\method{print}{getRanEf.BPREM}(x, ...)

\method{print}{getRanEf.CREM}(x, ...)
}
\arguments{
\item{x}{An object of class "BPREM" or "CREM".}

\item{...}{Additional arguments.}
}
\value{
Returns a list of the random effects for each individual/group.
}
\description{
Extracts the random effects from a fitted model of class "BPREM" or "CREM".
}
\examples{
# load fitted model results
data(results_bprem)
# get random effects
getRanEf(results_bprem)

}
\author{
Corissa T. Rohloff
}
