% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getVarCov.R
\name{getVarCov}
\alias{getVarCov}
\alias{getVarCov.BPREM}
\alias{getVarCov.CREM}
\alias{getVarCov.PREM}
\alias{print.getVarCov.BPREM}
\alias{print.getVarCov.CREM}
\alias{print.getVarCov.PREM}
\title{Extract random effects variance-covariance matrix}
\usage{
getVarCov(x, ...)

\method{getVarCov}{BPREM}(x, ...)

\method{getVarCov}{CREM}(x, ...)

\method{getVarCov}{PREM}(x, ...)

\method{print}{getVarCov.BPREM}(x, ...)

\method{print}{getVarCov.CREM}(x, ...)

\method{print}{getVarCov.PREM}(x, ...)
}
\arguments{
\item{x}{An object of class "BPREM", "CREM", or "PREM".}

\item{...}{Additional arguments.}
}
\value{
Returns a list of the random effects variance-covariance matrices.
}
\description{
Extracts the random effects variance-covariance matrix from a fitted model of class "BPREM", "CREM", or "PREM".
}
\examples{
# load fitted model results
data(results_prem)
# get random effects variance-covariance matrices
getVarCov(results_prem)

}
\author{
Corissa T. Rohloff
}
